#!/usr/bin/env python
#-*- coding: utf-8 -*-

import codecs
import os
import re
import sys

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    from configparser import RawConfigParser
    from io import StringIO
else:
    #Python2
    from ConfigParser import RawConfigParser
    from cStringIO import StringIO

import logging

from PySide import QtGui, QtCore

from .code2blocks import Code2Blocks
from .blocks import Blocks, OPENHANDCURSOR
from .work_area import WorkArea
from .tool_area import ToolArea
#from .constant import os.getenv("PINGUINO_NAME")
#from ..bloques.color import setColor
from ..py_bloques.get_blocks import all_sets
from ..bloques import BlockLinear, BlockFunction, BlockNested, BlockSpace, BlockNestedSecond, BlockSpaceBool, BlockFrameEdit
from ...ide.commons.backgrounds import BackgroundPallete
from ...ide.methods.decorators import Decorator
from ...frames.blocks_widget import Ui_widgetBlock
from ...frames.grafical_widget import Ui_Form_graphical
from ...ide.methods.dialogs import Dialogs

from ..py_bloques import constructor as BlocksConstructor



########################################################################
class GraphicalIDE(Code2Blocks):

    #----------------------------------------------------------------------
    def __init__(self, ide):

        self.main = ide.main
        self.ide = ide

        self.highlightedRect = QtCore.QRect()
        self.inPlace = 0
        self.posY = 20
        self.openID = 1

        self.fileName = None

        self.listBlocks = []
        #self.listSpaces = 0

        self.setup_bloques()

        self.main.tabWidget_files.setCurrentIndex(0)
        self.main.tabWidget_files.setMaximumSize(10000, 10e6)

        widget = self.get_widget("Pinguino")
        self.main.stackedWidget_blocks.setCurrentWidget(widget)

        self.constant_update()
        self.constant_auto_raise()


    # #----------------------------------------------------------------------
    # def set_tab(self, tab_name):
        # for index in range(self.main.tabWidget_blocks.count()):
            # if self.main.tabWidget_blocks.tabText(index) == tab_name:
                # self.main.tabWidget_blocks.setCurrentIndex(index)
                # break


    #----------------------------------------------------------------------
    def ide_new_file(self, *args, **kwargs):
        path = kwargs.get("filename", self.ide.get_untitled_name(ext=".gpde"))
        filename = os.path.split(path)[1]

        editor = QtGui.QWidget()
        widget = Ui_Form_graphical()
        widget.setupUi(editor)

        widget.widget.setMinimumSize(QtCore.QSize(2000, 2000))

        main_widget = widget.widget

        scroll_area = widget.scrollArea

        editor.scroll_area = widget.scrollArea

        area = WorkArea(parent=main_widget,
                        scroll=scroll_area,
                        #widget=widget,
                        frame=self,
                        ide=self.ide)

        editor.graphical_area = area

        self.main.tabWidget_files.addTab(editor, filename)

        editor.keyPressEvent = self.__keyPressEvent__

        self.main.tabWidget_files.setCurrentWidget(editor)
        index = self.main.tabWidget_files.currentIndex()
        self.main.tabWidget_files.setTabText(index, filename[:-1])
        self.ide.update_actions()


    #----------------------------------------------------------------------
    def __keyPressEvent__(self, event):
        editor = self.main.tabWidget_files.currentWidget()
        #editor.graphical_area

        if not editor.graphical_area.SelectArea.isVisible(): return

        if event.key() == QtCore.Qt.Key_Delete:
            editor.graphical_area.dele_blocks()


    # #----------------------------------------------------------------------
    # def ide_tab_changed(self, *args, **kwargs):
        # self.main.tabWidget_files.setVisible(self.main.tabWidget_files.count() > 0)
        # self.main.frame_logo.setVisible(not self.main.tabWidget_files.count() > 0)
        # self.main.actionClose_file.setEnabled(self.main.tabWidget_files.count() > 0)

        # editor = self.main.tabWidget_files.currentWidget()
        # if getattr(editor, "path", None): self.ide.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+editor.path)
        # else: self.ide.setWindowTitle(os.getenv("PINGUINO_NAME"))

        # index = self.main.tabWidget_files.currentIndex()
        # filename = self.main.tabWidget_files.tabText(index)
        # if filename.endswith("*"): self.main.actionSave_file.setEnabled(True)
        # else: self.main.actionSave_file.setDisabled(True)

        # self.ide.__update_current_dir_on_files__()


    #----------------------------------------------------------------------
    def ide_save_file(self, *args, **kwargs):

        editor = kwargs.get("editor", None)
        if not editor: editor = self.ide.get_tab().currentWidget()
        index = self.ide.get_tab().indexOf(editor)
        #editor = self.main.tabWidget_files.currentWidget()
        #index = self.main.tabWidget_files.currentIndex()
        filename = self.main.tabWidget_files.tabText(index)
        save_path = getattr(editor, "path", None)

        if not save_path:
            save_path, filename = Dialogs.set_save_file(self.ide, filename)
            if not save_path: return False
            setattr(editor, "path", save_path)
            self.main.tabWidget_files.setTabText(index, filename)
            self.main.tabWidget_files.setTabToolTip(index, save_path)
            self.ide.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+save_path)



        self.__save_file__(editor=editor)
        return True


    #----------------------------------------------------------------------
    def ide_save_as(self, *args, **kwargs):

        editor = kwargs.get("editor", None)
        if not editor: editor = self.ide.get_tab().currentWidget()
        index = self.ide.get_tab().indexOf(editor)
        #editor = self.main.tabWidget_files.currentWidget()
        #index = self.main.tabWidget_files.currentIndex()
        filename = self.main.tabWidget_files.tabText(index)
        save_path = getattr(editor, "path", None)

        save_path, filename = Dialogs.set_save_file(self.ide, filename)
        if not save_path: return False
        setattr(editor, "path", save_path)
        self.main.tabWidget_files.setTabText(index, filename)
        self.main.tabWidget_files.setTabToolTip(index, save_path)
        self.ide.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+save_path)

        self.__save_file__(editor=editor)
        return True



    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    def get_pinguino_source_code(self):

        editor = self.main.tabWidget_files.currentWidget()
        #editor.graphical_area.update_code()
        return editor.graphical_area.update_code()


    #----------------------------------------------------------------------
    def __save_file__(self, *args, **kwargs):

        editor = kwargs.get("editor", self.ide.get_tab())
        content = self.get_gpde(editor)
        self.save_raw_parser(content, editor.path)

        # setattr(editor, "last_saved", content)
        # self.ide.ide_remove_backup_file(editor=editor)

        self.ide.editor_saved()



    #----------------------------------------------------------------------
    def save_raw_parser(self, content, filename):

        file_parser = RawConfigParser()
        count = 0
        for block in content:
            count += 1
            name_section = "Block-%d"%count
            file_parser.add_section(name_section)
            for key in block.keys():
                file_parser.set(name_section, key, block[key])
        file_parser.write(codecs.open(filename, "w", encoding="utf-8"))


    #----------------------------------------------------------------------
    def read_raw_parser(self, filename):

        blocks_set = []
        file_parser = RawConfigParser()

        if os.getenv("PINGUINO_PYTHON") is "2":
            if type(filename) in [str, unicode]: file_parser.readfp(codecs.open(filename, "r", encoding="utf-8"))
            else: file_parser = filename
        elif os.getenv("PINGUINO_PYTHON") is "3":
            if type(filename) == str: file_parser.readfp(codecs.open(filename, "r", encoding="utf-8"))
            else: file_parser = filename

        sections = file_parser.sections()
        for section in sections:
            options = file_parser.options(section)
            block = {}
            for option in options:
                file_parser.get(section, option)
                value = file_parser.get(section, option)

                if os.getenv("PINGUINO_PYTHON") is "2":
                    if (type(value) in [str, unicode]) and (value[0] in ["[", "("]):
                        block[option] = eval(file_parser.get(section, option))
                    else:
                        block[option] = file_parser.get(section, option)
                elif os.getenv("PINGUINO_PYTHON") is "3":
                    if (type(value) == str) and (value[0] in ["[", "("]):
                        block[option] = eval(file_parser.get(section, option))
                    else:
                        block[option] = file_parser.get(section, option)

            blocks_set.append(block)
        return blocks_set


    #----------------------------------------------------------------------
    def get_gpde(self, editor=None):

        if editor is None:
            editor = self.main.tabWidget_files.currentWidget()

        blocks = editor.graphical_area.get_project_blocks()

        to_save = []
        for block in blocks:
            to_save.append({
                "basename": block.metadata.basename,
                "position": (block.metadata.pos_.x(), block.metadata.pos_.y()),
                #"position": (block.pos().x(), block.pos().y()),
                "constructor": block.metadata.get_contructor(),
                "name": block.metadata.name,
                "self_id": self.serialize_widgets([block])[0],
                "to": self.serialize_widgets(block.metadata.to),
                "from": self.serialize_widgets(block.metadata.from_),
                #"inside2": self.serialize_widgets(block.metadata.inside),
                "inside": self.serialize_widgets(self.sort_inside(block)),
                "nested": self.serialize_widgets(block.metadata.nested),
                })

        return to_save


    #----------------------------------------------------------------------
    def sort_inside(self, block):

        layout = block.metadata.object_.layout
        if layout is None: return []
        #bloque = block.ARGS[2:]
        #layout_pos = []
        #empty = block.metadata.object_.get_layout_pos_save()
        #fix = 0
        #for i in range(len(bloque)):
            #if bloque[i] in [["space"], ["space_bool"]]:
                #if i in empty:
                    #layout_pos.append(None)
                #else:
                    #layout_pos.append(True)
                    #fix += 1
                    #empty = map(lambda x:x-fix, empty)

        layout_pos = block.metadata.object_.get_layout_pos_save()

        widgets = []
        for i in range(layout.count()):
            widgets.append(layout.itemAt(i).widget())

        widgets = list(filter(lambda wdg:getattr(wdg, "metadata", False), widgets))

        inside = layout_pos[:]

        if inside.count(True) == len(widgets):
            for i in range(len(inside)):
                if inside[i]: inside[i] = widgets.pop(0)
        else:
            raise Exception("¬¬")

        return inside




    ##----------------------------------------------------------------------
    #def force_inside(self, block, inside):
        #



    #----------------------------------------------------------------------
    def serialize_widgets(self, widgets):

        ser = []
        for widget in widgets:
            #name = widget.__str__()
            if not widget:
                id_ = None
            else:
                id_ = re.findall("[.]*0x[\dabcdefABCDEF]*[.]*", widget.__str__())[0]
            ser.append(id_)
        return ser



    #----------------------------------------------------------------------
    def ide_open_files(self, *args, **kwargs):

        #filenames = Dialogs.set_open_file(self.ide, ".gpde")
        #for filename in filenames:

        filename = kwargs["filename"]

        if self.ide.ide_check_duplicate_file(filename): return

        self.ide_new_file(filename)
        editor = self.main.tabWidget_files.currentWidget()
        set_blocks = self.read_raw_parser(filename)
        self.load_blocks(set_blocks)
        self.main.tabWidget_files.setTabToolTip(self.main.tabWidget_files.currentIndex(), filename)
        self.main.tabWidget_files.setTabText(self.main.tabWidget_files.currentIndex(), os.path.split(filename)[1])
        self.ide.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+filename)
        setattr(editor, "path", filename)

        # self.ide.ide_tab_changed()

    #----------------------------------------------------------------------
    def load_blocks(self, set_blocks):

        editor = self.main.tabWidget_files.currentWidget()
        editor.graphical_area.isOpening = True

        toFitInside = []
        #restaurPos = []
        #all_icons = []

        #dummy, pos = editor.graphical_area.new_bloq("output", all_sets["convert_"], QtCore.QPoint(), "dummy_block", "dummy_block")
        #dummy.metadata.self_id = self.serialize_widgets([dummy])[0]

        for block in set_blocks:
            name = block["name"]
            args = ["", ""] + block["constructor"]
            pos = QtCore.QPoint(*block["position"])
            basename = block["basename"]
            ID = block["self_id"]

            newIcon = editor.graphical_area.new_bloq(name, args, pos, basename)[0]
            newIcon.metadata.self_id = ID

            newIcon.move(pos)
            newIcon.setCursor(QtGui.QCursor(OPENHANDCURSOR))

            newIcon.metadata.to = block["to"]
            newIcon.metadata.from_ = block["from"]
            newIcon.metadata.inside = block["inside"]
            #for i in range(newIcon.metadata.inside.count(None)):
                #newIcon.metadata.inside.remove(None)
            newIcon.metadata.nested = block["nested"]

        self.replace_id_by_widgets()

        for block in editor.graphical_area.get_project_blocks():
            if block.metadata.inside:
                toFitInside.append([block, block.metadata.inside[:]])
                #self.force_inside(block, block.metadata.inside)

        all_dummies = []
        for parent, toFit in toFitInside:
            for ins in toFit:
                wdg = self.get_widget_from_id(ins)
                if wdg:
                    wdg.metadata.add_parent([parent, wdg], force=True)
                else:
                    dummy, pos = editor.graphical_area.new_bloq("output", all_sets["convert_"], QtCore.QPoint(), "dummy_block", "dummy_block")
                    dummy.metadata.self_id = self.serialize_widgets([dummy])[0]
                    dummy.metadata.add_parent([parent, dummy], force=True)
                    all_dummies.append(dummy)
                    #dummy.

        for dummy in all_dummies:
            dummy.metadata.remove_parent()
            dummy.metadata.destroy_this()


        #for block in editor.graphical_area.get_project_blocks():
            #if block.metadata.type_ in "tipo9 tipo7".split(): block.lower()

        #for block in editor.graphical_area.get_project_blocks():
            #if block.metadata.type_ in "tipo4".split(): block.lower()





    #----------------------------------------------------------------------
    def replace_id_by_widgets(self):

        editor = self.main.tabWidget_files.currentWidget()
        for block in editor.graphical_area.get_project_blocks():
            block.metadata.to = list(map(self.get_widget_from_id, block.metadata.to))
            block.metadata.from_ = list(map(self.get_widget_from_id, block.metadata.from_))
            block.metadata.inside = list(map(self.get_widget_from_id, block.metadata.inside))
            block.metadata.nested = list(map(self.get_widget_from_id, block.metadata.nested))


    #----------------------------------------------------------------------
    def get_widget_from_id(self, id_):

        if type(id_) != type(""): return id_
        if id_ is None: return id_

        editor = self.main.tabWidget_files.currentWidget()
        blocks = editor.graphical_area.get_project_blocks()

        for block in blocks:
            if block.metadata.self_id == id_:
                return block


    #----------------------------------------------------------------------
    def ide_open_file_from_path(self, *args, **kwargs):

        filename = kwargs["filename"]

        if self.ide.ide_check_duplicate_file(filename): return
        self.ide_new_file(filename)
        editor = self.main.tabWidget_files.currentWidget()
        set_blocks = self.read_raw_parser(filename)
        self.load_blocks(set_blocks)

        editor = self.main.tabWidget_files.currentWidget()
        #index = self.main.tabWidget_files.currentIndex()
        #self.main.tabWidget_files.setTabText(index, os.path.split(filename)[1])
        #self.main.tabWidget_files.setTabToolTip(index, filename)
        editor.path = filename
        setattr(editor, "path", filename)
        self.main.tabWidget_files.setTabToolTip(self.main.tabWidget_files.currentIndex(), filename)
        self.main.tabWidget_files.setTabText(self.main.tabWidget_files.currentIndex(), os.path.split(filename)[1])
        self.ide.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+filename)

        # self.ide.ide_tab_changed()


    #----------------------------------------------------------------------
    def open_from_source(self, source):
        """"""
        filename = self.ide.get_untitled_name(ext=".gpde")

        self.ide_new_file(filename)
        editor = self.main.tabWidget_files.currentWidget()
        set_blocks = self.read_raw_parser(source)
        self.load_blocks(set_blocks)

        editor = self.main.tabWidget_files.currentWidget()
        editor.path = filename
        setattr(editor, "path", filename)
        self.main.tabWidget_files.setTabToolTip(self.main.tabWidget_files.currentIndex(), filename)
        self.main.tabWidget_files.setTabText(self.main.tabWidget_files.currentIndex(), os.path.split(filename)[1])
        self.ide.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+filename)

        # self.ide.ide_tab_changed()

        self.init_positions()


    #----------------------------------------------------------------------
    def init_positions(self):
        """"""
        editor = self.main.tabWidget_files.currentWidget()
        for block in editor.graphical_area.get_project_blocks():
            if block.metadata.from_:

                if block.metadata.name in ["output"]: continue

                if editor.graphical_area.get_type_magnetic(block, block.metadata.from_[0]):
                    pos = editor.graphical_area.get_type_magnetic(block, block.metadata.from_[0])[0][0]
                    block.move(pos)




    ##----------------------------------------------------------------------
    #def save_as_pde(self):

        #editor = self.main.tabWidget_files.currentWidget()
        #filename = editor.path.replace(".gpde", ".pde")
        #file_pde = codecs.open(filename, "w", encoding="utf-8")
        #file_pde.write(self.get_pinguino_source_code())
        #file_pde.close()


    #----------------------------------------------------------------------
    @Decorator.requiere_graphical_mode()
    def get_work_area(self):
        return self.main.tabWidget_files.currentWidget().graphical_area


    #----------------------------------------------------------------------
    def setup_bloques(self):
        self.StartBlocks = all_sets.copy()
        self.allTabs = {}

        self.build_block_tabs()

        self.main.comboBox_blocks.addItems(self.get_block_tabs())

        for tab in self.get_block_tabs():
            tab_set = []

            for key in all_sets.keys():
                if all_sets[key][1] == tab:
                    tab_set.append([key, all_sets[key]])

            self.add_blocks(tab, tab_set)


    #----------------------------------------------------------------------
    def get_tab_set(self, tab_name):
        tab_set = []
        for key in all_sets.keys():
            if all_sets[key][1] == tab_name:
                tab_set.append([key, all_sets[key]])
        return tab_set


    #----------------------------------------------------------------------
    def get_block_tabs(self):
        tabs = ["Search"]
        for key in all_sets.keys(): tabs.append(all_sets[key][1])
        tabs = list(set(tabs))
        tabs.remove("None")
        return tabs


    #----------------------------------------------------------------------
    def build_block_tabs(self, tabs=None):
        if tabs is None: tabs = self.get_block_tabs()
        for tab in tabs:
            widget = QtGui.QWidget()

            ui = Ui_widgetBlock()
            ui.setupUi(widget)

            tool_area = ToolArea(ui.scrollArea, self.get_work_area, self.ide)
            ui.gridLayout_2.addWidget(tool_area, 0, 0, 1, 1)

            # self.main.tabWidget_blocks.addTab(widget, tab)
            widget.setObjectName(tab)
            self.main.stackedWidget_blocks.addWidget(widget)

            ui.gridLayout_2.setContentsMargins(9, 9, 9, 9)
            ui.gridLayout_2.setSpacing(10)
            widget.grid_layout = ui.gridLayout_2
            widget.tool_area = tool_area
            tool_area.widget_parent = widget
            tool_area.scroll_area = ui.scrollArea

            #palette = QtGui.QPalette(self.main.centralwidget.parent().palette())
            #widget.setAutoFillBackground(True)
            #palette.setColor(QtGui.QPalette.Window, QtGui.QColor("#ffffff"))
            #widget.setPalette(palette)

            ui.frame.setStyleSheet("""

            QFrame{
                 background-color: #FFF;
             }

            """)

            ui.frame_2.setStyleSheet("""

            QFrame{
                 background-color: #FFF;
             }

            """)

            ui.scrollArea.setFrameShape(QtGui.QFrame.NoFrame)

            widget.mousePressEvent = tool_area.mouse_press_event
            widget.mouseMoveEvent = tool_area.mouse_move_event
            widget.mouseReleaseEvent = tool_area.mouse_release_event
            widget.content_widgets = []

    # #----------------------------------------------------------------------
    # def remove_tab(self, tab_name):
        # for index in range(self.main.tabWidget_blocks.count()):
            # if self.main.tabWidget_blocks.tabText(index) == tab_name:
                # self.main.tabWidget_blocks.removeTab(index)
                # break

    #----------------------------------------------------------------------
    def get_widget(self, tab_name):
        # for index in range(self.main.tabWidget_blocks.count()):
            # if self.main.tabWidget_blocks.tabText(index) == tab_name:
                # widget = self.main.tabWidget_blocks.widget(index)
                # return widget

        for index in range(self.main.stackedWidget_blocks.count()):
            # self.main.stackedWidget_blocks
            widget = self.main.stackedWidget_blocks.widget(index)
            if widget.objectName() == tab_name:
                return widget


    #----------------------------------------------------------------------
    def add_blocks(self, tab, tab_set, count=0, side=0, clear=False):
        widget = self.get_widget(tab)
        grid_layout = widget.grid_layout
        tool_area = widget.tool_area

        if clear: self.clear_area(tab)


        for key, tab in tab_set:

            newIcon = QtGui.QWidget(tool_area)
            eval(Blocks[tab[0]])(newIcon, tab[2:])
            newIcon.setMaximumSize(newIcon.size())

            widget.content_widgets.append(newIcon)

            newIcon.NAME = tab[0]
            newIcon.ARGS = tab
            newIcon.BASENAME = key

            grid_layout.addWidget(newIcon, count, side, 1, 1)
            count += 1




    # #----------------------------------------------------------------------
    # def add_blocks2(self, tab, tab_set):
        # # widget = self.get_widget(tab)
        # # grid_layout = widget.grid_layout
        # # tool_area = widget.tool_area

        # # if clear: self.clear_area(tab)

        # count = 0

        # for key, tab in tab_set:

            # area = self.ide.get_current_editor().graphical_area

            # newIcon = QtGui.QWidget(area)
            # eval(Blocks[tab[0]])(newIcon, tab[2:])
            # newIcon.setMaximumSize(newIcon.size())

            # # widget.content_widgets.append(newIcon)

            # newIcon.NAME = tab[0]
            # newIcon.ARGS = tab
            # newIcon.BASENAME = key

            # newIcon.move(10, 10)

            # # area.addWidget(newIcon, count, 0, 1, 1)
            # count += 1




    #----------------------------------------------------------------------
    def add_group_blocks(self, name, new_set):
        tab_set = self.get_tab_set(name)
        new_set = list(map(lambda x: [x[0], x[2]], new_set))
        self.add_blocks(name, tab_set + new_set)


    #----------------------------------------------------------------------
    def clear_area(self, area):

        widget = self.get_widget(area)

        for wid in widget.content_widgets:
            wid.deleteLater()
            widget.content_widgets.remove(wid)



    #----------------------------------------------------------------------
    def get_tab(self):

        return self.ide.get_tab()

    #----------------------------------------------------------------------
    def isActiveWindow(self):

        return self.ide.isActiveWindow()

    #----------------------------------------------------------------------
    def is_graphical(self):

        return self.ide.is_graphical()

    #----------------------------------------------------------------------
    def is_widget(self):
        """"""
        return self.ide.is_widget()



    #----------------------------------------------------------------------
    @Decorator.timer(50)
    @Decorator.requiere_open_files()
    @Decorator.requiere_graphical_mode()
    @Decorator.requiere_main_focus()
    def constant_update(self):
        if self.ide.is_graphical() is True:
            editor = self.main.tabWidget_files.currentWidget()
            editor.graphical_area.constant_update()


    #----------------------------------------------------------------------
    @Decorator.timer(500)
    @Decorator.requiere_open_files()
    @Decorator.requiere_graphical_mode()
    @Decorator.requiere_main_focus()
    def constant_auto_raise(self):

        editor = self.main.tabWidget_files.currentWidget()
        if not editor.graphical_area.isDragging:
            editor.graphical_area.auto_raise()


    #----------------------------------------------------------------------
    def update_blocks_search_tab(self, text):

        if len(text) < 2: return
        if text == QtGui.QApplication.translate("Frame", "Search Block..."): return
        if not text: return
        bloques = []
        exclude = []
        for key in all_sets.keys():
            if all_sets[key][2][0] in ["label", "decorator"]:
                label = all_sets[key][2][1]
                if label.lower().startswith(text.lower()):
                    bloques.append([key, all_sets[key]])
                    exclude.append(all_sets[key][2][1])

        bloques.extend(self.get_blocks_for_library(text.lower(), exclude))

        if not bloques: return

        self.clear_area("Search")
        self.add_blocks("Search", bloques)
        self.ide.set_block_tab("Search")


    #----------------------------------------------------------------------
    def get_all_sets(self):

        return all_sets


    #----------------------------------------------------------------------
    def get_blocks_for_library(self, text, exclude):
        """"""
        if not hasattr(self.ide, "assistant"): self.ide.set_assistant()

        blocks = []
        for item in self.ide.assistant:
            function = self.ide.assistant[item]
            if not item.lower().startswith(text): continue
            if item in exclude: continue

            if function["return"] == "void":
                block = BlocksConstructor.Linear

            elif function["return"] == "bool":
                block = BlocksConstructor.OutputBool

            else:
                block = BlocksConstructor.Output

            linecode = "{}({});".format(item, function["args"])
            blocks.append([item, self.get_contructor(block, linecode)])

        return blocks





#!/usr/bin/env python

from . import version

from .qtgui.pinguino_core import boards
from .qtgui.pinguino_core.config import Config
from .qtgui.pinguino_core.pinguino import Pinguino
from .qtgui.pinguino_core.pinguino_config import PinguinoConfig

#----------------------------------------------------------------------
def lazy(pinguino):
    """"""
    PinguinoConfig.set_environ_vars()
    PinguinoConfig.check_user_files()
    config = Config()
    PinguinoConfig.update_pinguino_paths(config, pinguino)
    PinguinoConfig.update_pinguino_extra_options(config, pinguino)
    PinguinoConfig.update_user_libs(pinguino)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
from ..methods.dialogs import Dialogs
from ..methods.decorators import Decorator

from PySide import QtGui, QtCore

########################################################################
class Files(object):

    #----------------------------------------------------------------------
    def __init__(self):
        """"""
        self.connect(self.main.comboBox_files, QtCore.SIGNAL("currentIndexChanged(int)"), self.change_dir_files)
        self.connect(self.main.treeWidget_explorer, QtCore.SIGNAL("itemExpanded(QTreeWidgetItem*)"), self.expand_tree)
        self.connect(self.main.treeWidget_explorer, QtCore.SIGNAL("itemDoubleClicked(QTreeWidgetItem*,int)"), self.open_from_tree)


    #----------------------------------------------------------------------
    def update_path_files(self, path):

        if not type(path) is list: path = [path]

        flags = QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsDragEnabled|QtCore.Qt.ItemIsUserCheckable|QtCore.Qt.ItemIsEnabled
        self.main.treeWidget_explorer.clear()
        for dir_project in path:
            if os.path.exists(dir_project):
                if os.listdir(dir_project):
                    parent = self.add_new_tree(os.path.basename(dir_project), self.main.treeWidget_explorer, dir_project, flags)
                    self.generate_tree(dir_project, parent, levels=1, flags=flags)
                    parent.setExpanded(True)


    #----------------------------------------------------------------------
    def generate_tree(self, path, parent, levels=0, currentlevel=1, flags=None, to_ignore=[], inherits_status=dict(), library=False):

        if currentlevel > levels:
            if os.listdir(path):
                self.add_new_file_item("empty", parent, "empty", flags=None)
            return

        if not os.path.exists(path):
            return

        lisdir = os.listdir(path)

        # Filters
        lisdir = filter(lambda f:not f.startswith("."), lisdir)
        lisdir = filter(lambda f:not f.endswith(".ppde"), lisdir)
        lisdir = filter(lambda f:not f.endswith(".hex"), lisdir)
        lisdir = filter(lambda f:not f.endswith("~"), lisdir)

        list_dirs = []
        list_files = []

        for item in lisdir:
            fullpath = os.path.join(path, item)

            if fullpath in to_ignore: continue
            # print path in to_ignore, path, to_ignore

            if os.path.isdir(fullpath):
                list_dirs.append(fullpath)
            elif os.path.isfile(fullpath):
                list_files.append(fullpath)

        list_dirs.sort()
        list_files.sort()

        files_from_tree = []
        for dir_ in list_dirs:
            dir_name = os.path.basename(dir_)
            ft = self.generate_tree(dir_, self.add_new_tree(dir_name, parent, dir_, flags), currentlevel=currentlevel+1, levels=levels, to_ignore=to_ignore, inherits_status=inherits_status, library=library, flags=flags)
            if ft: files_from_tree.extend(ft)

        for file_ in list_files:
            file_name = os.path.basename(file_)
            if inherits_status and type(inherits_status) == dict:
                if file_ in inherits_status: check = inherits_status[file_]
                else: check = False
            elif type(inherits_status) == bool:
                check = inherits_status
            else:
                check = None

            if (file_name.endswith(".pde") or file_name.endswith(".h") or file_name.endswith(".c") or file_name.endswith(".cpp")) and (library is False):
                self.add_new_file_item(file_name, parent, file_, flags, check)
            elif library:
                self.add_new_file_item(file_name, parent, file_, None, None)
            else:
                self.add_new_file_item(file_name, parent, file_, flags, None)


        list_files.extend(files_from_tree)
        list_files.sort()

        return list_files


    #----------------------------------------------------------------------
    def add_new_file_item(self, filename, parent, fullpath, flags=None, check=None):
        """"""
        file_item = QtGui.QTreeWidgetItem(parent)

        file_item.setText(0, filename)
        file_item.path = fullpath


        if filename.endswith(".py"):
            file_item.setIcon(0, QtGui.QIcon.fromTheme("file-python"))
        elif filename.endswith(".pde"):
            file_item.setIcon(0, QtGui.QIcon.fromTheme("file-csrc"))
        elif filename.endswith(".h"):
            file_item.setIcon(0, QtGui.QIcon.fromTheme("file-chdr"))
        else:
            file_item.setIcon(0, QtGui.QIcon.fromTheme("document-new"))

        if flags:
            file_item.setFlags(flags)

        if not check is None:
            if check:
                file_item.setCheckState(0, QtCore.Qt.Checked)
            else:
                file_item.setCheckState(0, QtCore.Qt.Unchecked)



    #----------------------------------------------------------------------
    def add_new_tree(self, name, parent, fullpath, flags=None):
        """"""
        tree = QtGui.QTreeWidgetItem(parent)
        tree.setText(0, name)
        tree.path = fullpath
        # icon_dir = QtGui.QIcon()
        # icon_dir.addPixmap(QtGui.QPixmap(":/icons/icons/icon_dir.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon_dir = QtGui.QIcon.fromTheme("document-open")

        tree.setIcon(0, icon_dir)

        if flags:
            tree.setFlags(flags)

        return tree


    #----------------------------------------------------------------------
    def expand_tree(self, item):
        """"""
        dir_name = item.text(0)
        dir_ = item.path
        parent = item

        if item.child(0) and item.child(0).path == "empty":
            self.generate_tree(dir_, parent, levels=1)
            item.removeChild(item.child(0))


    #----------------------------------------------------------------------
    def open_from_tree(self, event, int_):

        if os.path.isfile(event.path):
            self.ide_open_file_from_path(filename=event.path)


    #----------------------------------------------------------------------
    def jump_dir_files(self, list_widget_item):
        if getattr(list_widget_item, "type_file") == "dir":
            self.update_path_files(getattr(list_widget_item, "path_file"))
        if getattr(list_widget_item, "type_file") == "file":
            self.ide_open_file_from_path(filename=getattr(list_widget_item, "path_file"))


    #----------------------------------------------------------------------
    def change_dir_files(self, index):
        to_dir = ["Examples", "Dir", "Home", "Other"][index]

        if to_dir == "Examples":
            self.update_path_files([os.path.join(os.getenv("PINGUINO_USER_PATH"), "examples"),
                                    os.path.join(os.getenv("PINGUINO_USER_PATH"), "graphical_examples"),
                                    os.path.join(os.getenv("PINGUINO_USER_PATH"), "examples_libs"),
                                    ])

        elif to_dir == "Home":
            self.update_path_files(QtCore.QDir.home().path())

        elif to_dir == "Dir":
            editor = self.get_current_editor()
            dir_ = getattr(editor, "path", None)
            if dir_: self.update_path_files(os.path.split(dir_)[0])

        # elif to_dir == "Third":
            # path = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "examples")
            # if os.path.exists(path): self.update_path_files(path)

        elif to_dir == "Other":
            open_dir = Dialogs.set_open_dir(self)
            if open_dir:
                self.update_path_files(open_dir)




#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide import QtGui, QtCore

from ..methods.decorators import Decorator


########################################################################
class Search(object):

    #----------------------------------------------------------------------
    def __init__(self):

        self.connect(self.main.actionSearch, QtCore.SIGNAL("triggered()"), lambda :self.set_tab_search("search"))
        self.connect(self.main.actionSearch_and_replace, QtCore.SIGNAL("triggered()"), lambda :self.set_tab_search("replace"))

        self.connect(self.main.pushButton_search_previous, QtCore.SIGNAL("clicked()"), self.search_previous)
        self.connect(self.main.pushButton_search_next, QtCore.SIGNAL("clicked()"), self.search_next)
        self.connect(self.main.pushButton_replace, QtCore.SIGNAL("clicked()"), self.replace)
        self.connect(self.main.pushButton_replace_all, QtCore.SIGNAL("clicked()"), self.replaceall)
        self.connect(self.main.lineEdit_search, QtCore.SIGNAL("textChanged(QString)"), self.search_instantaneous)
        self.connect(self.main.lineEdit_replace, QtCore.SIGNAL("textChanged(QString)"), self.replace_instantaneous)

        self.main.pushButton_search_next.setIcon(QtGui.QIcon.fromTheme("go-down"))
        self.main.pushButton_search_previous.setIcon(QtGui.QIcon.fromTheme("go-top"))

        # icon = QtGui.QIcon()
        # icon.addPixmap(QtGui.QIcon.fromTheme("insert-text").pixmap(size), QtGui.QIcon.Normal, QtGui.QIcon.On)
        # icon.addPixmap(QtGui.QIcon.fromTheme("insert-object").pixmap(size), QtGui.QIcon.Normal, QtGui.QIcon.Off)



    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    @Decorator.requiere_text_mode()
    @Decorator.requiere_line_edit_content("main.lineEdit_replace")
    def replace_instantaneous(self, text_to_replace):
        empty = not text_to_replace == QtGui.QApplication.translate("Frame", "Replace...")
        self.main.pushButton_replace.setEnabled(empty)
        self.main.pushButton_replace_all.setEnabled(empty)
        if empty: return


    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    @Decorator.requiere_text_mode()
    @Decorator.requiere_line_edit_content("main.lineEdit_search")
    def search_instantaneous(self, text_to_search):
        empty = not text_to_search == QtGui.QApplication.translate("Frame", "Search...")
        self.main.pushButton_search_previous.setEnabled(empty)
        self.main.pushButton_search_next.setEnabled(empty)

        editor = self.get_current_editor()
        editor.text_edit.setExtraSelections([])
        if not empty: return

        if self.main.checkBox_case_sensitive.isChecked():
            content = editor.text_edit.toPlainText()
            count = content.count(text_to_search)
        else:
            content = editor.text_edit.toPlainText().lower()
            count = content.count(text_to_search.lower())

        if count == 0: message = QtGui.QApplication.translate("Frame", "No words were found.")
        elif  count == 1: message = QtGui.QApplication.translate("Frame", "One word were found.")
        else: message = QtGui.QApplication.translate("Frame", "{} words were found.".format(count))
        self.main.label_replace_info.setText(message)

        #for i in range(count): self.search_next()

        text_cur = editor.text_edit.textCursor()
        editor.text_edit.moveCursor(text_cur.Start)
        self.__search__(text_to_search)



    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    @Decorator.requiere_line_edit_content("main.lineEdit_search")
    def __search__(self, text_to_search):
        if text_to_search == QtGui.QApplication.translate("Frame", "Search..."): return


        self.find_match(word=text_to_search,
                        back=False,
                        sensitive=self.main.checkBox_case_sensitive.isChecked(),
                        whole=False)

    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    @Decorator.requiere_text_mode()
    @Decorator.requiere_line_edit_content("main.lineEdit_search")
    def search_previous(self):
        self.find_match(word=self.main.lineEdit_search.text(),
                        back=True,
                        sensitive=self.main.checkBox_case_sensitive.isChecked(),
                        whole=False)

    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    @Decorator.requiere_text_mode()
    @Decorator.requiere_line_edit_content("main.lineEdit_search")
    def search_next(self):
        self.find_match(word=self.main.lineEdit_search.text(),
                        back=False,
                        sensitive=self.main.checkBox_case_sensitive.isChecked(),
                        whole=False)

    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    @Decorator.requiere_text_mode()
    @Decorator.requiere_line_edit_content("main.lineEdit_search")
    @Decorator.requiere_line_edit_content("main.lineEdit_replace")
    def replace(self):
        editor = self.get_current_editor()
        text_cur = editor.text_edit.textCursor()
        if text_cur.selectedText() == self.main.lineEdit_search.text():
            text_cur.removeSelectedText()
            text_cur.insertText(self.main.lineEdit_replace.text())
            text_cur.clearSelection()
            self.editor_highligh_line(line=None, color="#ffff7f")

    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    @Decorator.requiere_text_mode()
    @Decorator.requiere_line_edit_content("main.lineEdit_search")
    @Decorator.requiere_line_edit_content("main.lineEdit_replace")
    def replaceall(self):
        self.replace_all_match(wordOld=self.main.lineEdit_search.text(),
                               wordNew=self.main.lineEdit_replace.text(),
                               sensitive=self.main.checkBox_case_sensitive.isChecked(),
                               whole=False)

    #----------------------------------------------------------------------
    def replace_all_match(self, wordOld, wordNew, sensitive=False, whole=False):
        editor = self.get_current_editor()
        text_doc = editor.text_edit.document()
        text_cur = editor.text_edit.textCursor()
        s = text_doc.FindCaseSensitively if sensitive else 0
        w = text_doc.FindWholeWords if whole else 0
        # editor.text_edit.moveCursor(text_cur.NoMove, text_cur.KeepAnchor)
        text_cur.beginEditBlock()
        editor.text_edit.moveCursor(text_cur.Start)
        count = 0
        while True:
            result = False
            if sensitive or whole: result = editor.text_edit.find(wordOld, s | w)
            else: result = editor.text_edit.find(wordOld)
            if result:
                tc = editor.text_edit.textCursor()
                if tc.hasSelection(): tc.insertText(wordNew)
                count += 1
            else: break
            #replace = False
        text_cur.endEditBlock()
        self.main.label_replace_info.setText("{} words were replaced.".format(count))

    #----------------------------------------------------------------------
    def find_match(self, word, back, sensitive, whole):
        editor = self.get_current_editor()
        text_doc = editor.text_edit.document()
        text_cur = editor.text_edit.textCursor()
        b = text_doc.FindBackward if back else 0
        s = text_doc.FindCaseSensitively if sensitive else 0
        w = text_doc.FindWholeWords if whole else 0
        # editor.text_edit.moveCursor(text_cur.NoMove, text_cur.KeepAnchor)
        if back or sensitive or whole: editor.text_edit.find(word, b | s | w)
        else:  editor.text_edit.find(word)
        self.editor_highligh_line(line=None, color="#ffff7f", text_cursor=editor.text_edit.textCursor())





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/frames/library_template.ui'
#
# Created: Wed Mar 16 13:19:36 2016
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_LibraryTemplate(object):
    def setupUi(self, LibraryTemplate):
        LibraryTemplate.setObjectName("LibraryTemplate")
        LibraryTemplate.resize(518, 276)
        self.gridLayout = QtGui.QGridLayout(LibraryTemplate)
        self.gridLayout.setObjectName("gridLayout")
        self.pushButton_cancel = QtGui.QPushButton(LibraryTemplate)
        self.pushButton_cancel.setObjectName("pushButton_cancel")
        self.gridLayout.addWidget(self.pushButton_cancel, 1, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 0, 1, 1)
        self.formLayout = QtGui.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtGui.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label_2 = QtGui.QLabel(LibraryTemplate)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(0, QtGui.QFormLayout.LabelRole, self.label_2)
        self.lineEdit_lib_author = QtGui.QLineEdit(LibraryTemplate)
        self.lineEdit_lib_author.setObjectName("lineEdit_lib_author")
        self.formLayout.setWidget(0, QtGui.QFormLayout.FieldRole, self.lineEdit_lib_author)
        self.label_4 = QtGui.QLabel(LibraryTemplate)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(1, QtGui.QFormLayout.LabelRole, self.label_4)
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lineEdit_lib_path = QtGui.QLineEdit(LibraryTemplate)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_lib_path.sizePolicy().hasHeightForWidth())
        self.lineEdit_lib_path.setSizePolicy(sizePolicy)
        self.lineEdit_lib_path.setReadOnly(True)
        self.lineEdit_lib_path.setObjectName("lineEdit_lib_path")
        self.horizontalLayout_3.addWidget(self.lineEdit_lib_path)
        self.pushButton_lib_path = QtGui.QPushButton(LibraryTemplate)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton_lib_path.sizePolicy().hasHeightForWidth())
        self.pushButton_lib_path.setSizePolicy(sizePolicy)
        self.pushButton_lib_path.setMinimumSize(QtCore.QSize(0, 0))
        self.pushButton_lib_path.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pushButton_lib_path.setObjectName("pushButton_lib_path")
        self.horizontalLayout_3.addWidget(self.pushButton_lib_path)
        self.formLayout.setLayout(1, QtGui.QFormLayout.FieldRole, self.horizontalLayout_3)
        self.label = QtGui.QLabel(LibraryTemplate)
        self.label.setObjectName("label")
        self.formLayout.setWidget(2, QtGui.QFormLayout.LabelRole, self.label)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lineEdit_lib_name = QtGui.QLineEdit(LibraryTemplate)
        self.lineEdit_lib_name.setObjectName("lineEdit_lib_name")
        self.horizontalLayout_2.addWidget(self.lineEdit_lib_name)
        self.checkBox_lib8 = QtGui.QCheckBox(LibraryTemplate)
        self.checkBox_lib8.setChecked(True)
        self.checkBox_lib8.setObjectName("checkBox_lib8")
        self.horizontalLayout_2.addWidget(self.checkBox_lib8)
        self.checkBox_lib32 = QtGui.QCheckBox(LibraryTemplate)
        self.checkBox_lib32.setObjectName("checkBox_lib32")
        self.horizontalLayout_2.addWidget(self.checkBox_lib32)
        self.formLayout.setLayout(2, QtGui.QFormLayout.FieldRole, self.horizontalLayout_2)
        self.label_3 = QtGui.QLabel(LibraryTemplate)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(3, QtGui.QFormLayout.LabelRole, self.label_3)
        self.plainTextEdit_lib_description = QtGui.QPlainTextEdit(LibraryTemplate)
        self.plainTextEdit_lib_description.setObjectName("plainTextEdit_lib_description")
        self.formLayout.setWidget(3, QtGui.QFormLayout.FieldRole, self.plainTextEdit_lib_description)
        self.gridLayout.addLayout(self.formLayout, 0, 0, 1, 4)
        self.pushButton_accept = QtGui.QPushButton(LibraryTemplate)
        self.pushButton_accept.setEnabled(False)
        self.pushButton_accept.setObjectName("pushButton_accept")
        self.gridLayout.addWidget(self.pushButton_accept, 1, 2, 1, 1)

        self.retranslateUi(LibraryTemplate)
        QtCore.QMetaObject.connectSlotsByName(LibraryTemplate)

    def retranslateUi(self, LibraryTemplate):
        LibraryTemplate.setWindowTitle(QtGui.QApplication.translate("LibraryTemplate", "Library Template", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_cancel.setText(QtGui.QApplication.translate("LibraryTemplate", "Cancel", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("LibraryTemplate", "Author:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("LibraryTemplate", "Create in (*):", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_lib_path.setText(QtGui.QApplication.translate("LibraryTemplate", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("LibraryTemplate", "Library name (*):", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox_lib8.setText(QtGui.QApplication.translate("LibraryTemplate", "8bit", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox_lib32.setText(QtGui.QApplication.translate("LibraryTemplate", "32bit", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("LibraryTemplate", "Description:", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_accept.setText(QtGui.QApplication.translate("LibraryTemplate", "Continue", None, QtGui.QApplication.UnicodeUTF8))




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import sys
import re

from PySide import QtGui, QtCore

from ..methods.dialogs import Dialogs
from ...frames.library_template import Ui_LibraryTemplate


########################################################################
class LibraryTemplate(QtGui.QDialog):

    def __init__(self, parent):
        super(LibraryTemplate, self).__init__()

        self.lib_template = Ui_LibraryTemplate()
        self.lib_template.setupUi(self)
        self.main = parent

        self.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+self.windowTitle())

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/logo/art/windowIcon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)

        self.lib_template.lineEdit_lib_author.setText(os.environ.get("USER", ""))

        self.connect(self.lib_template.pushButton_accept, QtCore.SIGNAL("clicked()"), self.accept)
        self.connect(self.lib_template.pushButton_cancel, QtCore.SIGNAL("clicked()"), self.close)
        self.connect(self.lib_template.pushButton_lib_path, QtCore.SIGNAL("clicked()"), self.set_directory)
        self.connect(self.lib_template.checkBox_lib32, QtCore.SIGNAL("clicked(bool)"), self.can_generate)
        self.connect(self.lib_template.checkBox_lib8, QtCore.SIGNAL("clicked(bool)"), self.can_generate)
        self.connect(self.lib_template.lineEdit_lib_name, QtCore.SIGNAL("textChanged(QString)"), self.can_generate)
        self.connect(self.lib_template.lineEdit_lib_path, QtCore.SIGNAL("textChanged(QString)"), self.can_generate)

        self.center_on_screen()


    #----------------------------------------------------------------------
    def center_on_screen(self):

        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)


    #----------------------------------------------------------------------
    def set_directory(self):

        target_dir = Dialogs.set_open_dir(self.main)
        if target_dir:
            self.lib_template.lineEdit_lib_path.setText(target_dir)


    #----------------------------------------------------------------------
    def accept(self):

        author = self.lib_template.lineEdit_lib_author.text()
        libname = self.lib_template.lineEdit_lib_name.text()
        libpath = self.lib_template.lineEdit_lib_path.text()
        description = self.lib_template.plainTextEdit_lib_description.toPlainText()
        _8bit = self.lib_template.checkBox_lib8.isChecked()
        _32bit = self.lib_template.checkBox_lib32.isChecked()

        kwargs = locals().copy()
        kwargs.pop("self")
        self.main.new_library(**kwargs)
        self.close()


    #----------------------------------------------------------------------
    def can_generate(self, event):

        libname = self.lib_template.lineEdit_lib_name.text()
        libpath = self.lib_template.lineEdit_lib_path.text()
        _8bit = self.lib_template.checkBox_lib8.isChecked()
        _32bit = self.lib_template.checkBox_lib32.isChecked()

        if re.match("[a-zA-Z_]+[_0-9a-zA-Z]*", libname):
            self.lib_template.pushButton_accept.setEnabled(libname and os.path.isdir(libpath) and (_8bit or _32bit))
        else:
            self.lib_template.pushButton_accept.setEnabled(False)


#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import codecs

from PySide import QtGui

from .core_threads import UpdateAutocompleter
from ..methods.decorators import Decorator
# from ..tools.code_navigator import self
from ..methods.dialogs import Dialogs

########################################################################
class TimedMethods(object):



    #----------------------------------------------------------------------
    @Decorator.timer(1500)
    @Decorator.requiere_open_files()
    @Decorator.requiere_browser_tab("Functions")
    @Decorator.requiere_tools_tab("SourceBrowser")
    @Decorator.requiere_main_focus()
    def timer_update_functions(self):

        functions_parse = self.get_functions()

        index = 0
        self.main.tableWidget_functions.setRowCount(len(functions_parse))
        self.completer_funtions = []
        for funtion in functions_parse:
            item = QtGui.QTableWidgetItem()
            self.main.tableWidget_functions.setVerticalHeaderItem(index, item)
            item.setText(funtion["name"])
            self.completer_funtions.append(funtion["name"])

            self.main.tableWidget_functions.setItem(index, 0, QtGui.QTableWidgetItem())
            self.main.tableWidget_functions.setItem(index, 1, QtGui.QTableWidgetItem())
            self.main.tableWidget_functions.setItem(index, 2, QtGui.QTableWidgetItem())

            if funtion["filename"]:
                self.main.tableWidget_functions.item(index, 0).setText(os.path.split(funtion["filename"])[1])
            else:
                self.main.tableWidget_functions.item(index, 0).setText(self.get_current_filename())


            self.main.tableWidget_functions.item(index, 1).setText(funtion["return"])
            self.main.tableWidget_functions.item(index, 2).setText(funtion["args"])

            setattr(item, "filename", funtion["filename"])
            setattr(item, "line", funtion["line"])

            index += 1


    #----------------------------------------------------------------------
    @Decorator.timer(1500)
    @Decorator.requiere_open_files()
    @Decorator.requiere_browser_tab("Directives")
    @Decorator.requiere_tools_tab("SourceBrowser")
    @Decorator.requiere_main_focus()
    def timer_update_directives(self):
        """"""

        directives_parse = self.get_directives()

        index = 0
        self.main.tableWidget_directives.setRowCount(len(directives_parse))
        self.completer_directives = []
        for directive in directives_parse:
            item = QtGui.QTableWidgetItem()
            self.main.tableWidget_directives.setVerticalHeaderItem(index, item)
            item.setText(directive["name"])
            self.completer_directives.append(directive["name"])

            self.main.tableWidget_directives.setItem(index, 0, QtGui.QTableWidgetItem())
            self.main.tableWidget_directives.setItem(index, 1, QtGui.QTableWidgetItem())
            self.main.tableWidget_directives.setItem(index, 2, QtGui.QTableWidgetItem())

            if directive["filename"]:
                self.main.tableWidget_directives.item(index, 0).setText(os.path.split(directive["filename"])[1])
            else:
                self.main.tableWidget_directives.item(index, 0).setText(self.get_current_filename())

            self.main.tableWidget_directives.item(index, 1).setText(directive["type"])
            self.main.tableWidget_directives.item(index, 2).setText(directive["value"])

            setattr(item, "filename", directive["filename"])
            setattr(item, "line", directive["line"])
            index += 1



    #----------------------------------------------------------------------
    @Decorator.timer(1500)
    @Decorator.requiere_open_files()
    @Decorator.requiere_browser_tab("Variables")
    @Decorator.requiere_tools_tab("SourceBrowser")
    @Decorator.requiere_main_focus()
    def timer_update_variables(self):

        variables_parse = self.get_variables()

        index = 0
        self.main.tableWidget_variables.setRowCount(len(variables_parse))
        self.completer_variables = []
        for variable in variables_parse:
            item = QtGui.QTableWidgetItem()
            self.main.tableWidget_variables.setVerticalHeaderItem(index, item)
            item.setText(variable["name"])
            self.completer_variables.append([variable["name"], variable["type"]])

            self.main.tableWidget_variables.setItem(index, 0, QtGui.QTableWidgetItem())
            self.main.tableWidget_variables.setItem(index, 1, QtGui.QTableWidgetItem())

            if variable["filename"]:
                self.main.tableWidget_variables.item(index, 0).setText(os.path.split(variable["filename"])[1])
            else:
                self.main.tableWidget_variables.item(index, 0).setText(self.get_current_filename())

            self.main.tableWidget_variables.item(index, 1).setText(variable["type"])

            setattr(item, "filename", variable["filename"])
            setattr(item, "line", variable["line"])
            index += 1


    #----------------------------------------------------------------------
    @Decorator.timer(3000)
    @Decorator.requiere_open_files()
    @Decorator.requiere_text_mode()
    @Decorator.requiere_main_focus()
    def timer_check_changes(self):

        editor = self.get_current_editor()
        filename = getattr(editor, "path", None)
        if not filename:
            return

        if os.path.exists(filename):
            file_ = codecs.open(filename, "r", encoding="utf-8")
            content_file = "".join(file_.readlines())
            file_.close()
            exist = True
        else:
            content_file = ""
            exist = False

        last_saved = getattr(editor, "last_saved")

        # if content_file != last_saved:
            # self.thread_variables()

        if self.get_current_filename().endswith("(r/o)"):
            if content_file != last_saved:
                self.editor_reload_file()
            self.ide_save_file()
            return

        if self.is_library():
            name = self.get_project_name()
            if self.get_current_filename().startswith(name) and (self.get_current_filename().endswith(".c")\
                                                                 or self.get_current_filename().endswith(".h")\
                                                                 or self.get_current_filename().endswith(".pdl")\
                                                                 or self.get_current_filename().endswith(".pdl32")):
                if content_file != last_saved:
                    self.editor_reload_file()
                self.ide_save_file()
                return




        if content_file != last_saved:
            reload_ = Dialogs.overwrite_file(self, filename, exist)

            if reload_:
                self.ide_save_file()
            else:
                self.editor_reload_file()


    #----------------------------------------------------------------------
    @Decorator.timer(3000)
    @Decorator.requiere_open_files()
    # @Decorator.requiere_text_mode()
    @Decorator.requiere_main_focus()
    def timer_backup_file(self):

        editor = self.get_current_editor()
        # index = self.main.tabWidget_files.indexOf(editor)
        # filename_tab = self.main.tabWidget_files.tabText(index)
        filename = getattr(editor, "path", None)
        if not filename:
            return
        content_saved = getattr(editor, "last_saved", None)

        if self.is_graphical() is True:
            content = self.PinguinoKIT.get_gpde()
            self.PinguinoKIT.save_raw_parser(content, editor.path+"~")

        elif self.is_graphical() is False:
            content = editor.text_edit.toPlainText()
            filename_backup = filename + "~"
            #if os.path.exists(filename) and filename_tab.endswith("*"):
            if os.path.exists(filename) and (content_saved != content):
                file_ = codecs.open(filename_backup, "w", encoding="utf-8")
                file_.write(content)
                file_.close()

        elif content_saved == content and os.path.exists(filename_backup):
            os.remove(filename_backup)


    #----------------------------------------------------------------------
    @Decorator.timer(1000)
    @Decorator.requiere_open_files()
    @Decorator.requiere_tools_tab("SourceBrowser")
    @Decorator.requiere_main_focus()
    def timer_update_assiatant(self):

        if not self.is_graphical() is None:
            editor = self.get_current_editor()
            if not hasattr(editor, "text_edit"): return
            tc = editor.text_edit.textCursor()
            tc.movePosition(tc.EndOfWord, tc.MoveAnchor)
            editor.text_edit.smart_under_selection(tc)
            selected = tc.selectedText()
            self.update_assistant(selected)


    #----------------------------------------------------------------------
    @Decorator.timer(3000)
    @Decorator.requiere_open_files()
    @Decorator.requiere_text_mode()
    @Decorator.if_autocomplete_is_enable()
    @Decorator.requiere_main_focus()
    def timer_update_autocompleter(self):
        """"""

        if not self.is_graphical() is False: return

        editor = self.get_current_editor()

        if (not hasattr(self, "thread_autocompleter")) and hasattr(editor.text_edit, "completer"):
            self.thread_autocompleter = UpdateAutocompleter()
            self.thread_autocompleter.signal_set_variables.connect(self.set_variables)
            self.thread_autocompleter.signal_set_directives.connect(self.set_directives)
            self.thread_autocompleter.signal_set_functions.connect(self.set_functions)
            self.thread_autocompleter.signal_add_autocompleter.connect(editor.text_edit.completer.addTemporalItem)
            self.thread_autocompleter.signal_rm_autocompleter.connect(editor.text_edit.completer.removeTemporalItems)

        if not hasattr(self, "thread_autocompleter"):
            return

        if not self.thread_autocompleter.isRunning():
            self.thread_autocompleter.set_files(self.get_files_to_explore())
            self.thread_autocompleter.setTerminationEnabled(True)
            self.thread_autocompleter.start()
        ##else:
            ##logging.debug("Working...")
            ##self.thread_autocompleter.terminate()
            ##self.thread_autocompleter.set_files(self.get_files_to_explore())
            ##self.thread_autocompleter.setTerminationEnabled(True)
            ##self.thread_autocompleter.start()




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import sys
import os
import platform
import inspect
import types

from PySide import QtGui, QtCore

from .commons.backgrounds import BackgroundPallete
from .events import PinguinoEvents
from .methods.decorators import Decorator
from .custom_widgets.code_editor.autocomplete_icons import CompleteIcons
from .custom_widgets.code_editor.autocompleter import PinguinoAutoCompleter
# from .custom_widgets import PinguinoTextEdit
#from .help.help import Help
# from .methods.widgets_features import PrettyFeatures
from ..gide.app.graphical import GraphicalIDE
from ..pinguino_core.pinguino import Pinguino, AllBoards
from ..pinguino_core.pinguino_config import PinguinoConfig
from ..pinguino_core.config import Config
from ..frames.main import Ui_PinguinoIDE

import logging
#import debugger

from .tools.python_shell import PythonShell
from .tools.log import Log
from .tools.stdout import Stdout

########################################################################
class PinguinoIDE(QtGui.QMainWindow, PinguinoEvents, PythonShell, Log, Stdout):

    #@Decorator.debug_time()
    def __init__(self, splash_write):
        super(PinguinoIDE, self).__init__()

        os.environ["PINGUINO_PROJECT"] = ""

        QtCore.QTextCodec.setCodecForCStrings(QtCore.QTextCodec.codecForName("UTF-8"))
        QtCore.QTextCodec.setCodecForLocale(QtCore.QTextCodec.codecForName("UTF-8"))
        QtCore.QTextCodec.setCodecForTr(QtCore.QTextCodec.codecForName("UTF-8"))

        self.main = Ui_PinguinoIDE()
        self.main.setupUi(self)

        # self.argvs = argvs

        # if not self.argvs.devmode:
            # self.main.menubar.removeAction(self.main.menuDevelopment.menuAction())

        splash_write(QtGui.QApplication.translate("Splash", "Setting enviroment values"))
        PinguinoConfig.set_environ_vars()
        splash_write(QtGui.QApplication.translate("Splash", "Checking user files"))
        PinguinoConfig.check_user_files()

        splash_write(QtGui.QApplication.translate("Splash", "Loading Pinguino Core"))
        self.pinguinoAPI = Pinguino()
        self.pinguinoAPI._boards_ = AllBoards

        splash_write(QtGui.QApplication.translate("Splash", "Loading configuration"))
        self.configIDE = Config()

        splash_write(QtGui.QApplication.translate("Splash", "Loading blocks programming"))
        self.PinguinoKIT = GraphicalIDE(self)
        self.main.tabWidget_files.setVisible(False)
        # self.main.dockWidget_blocks.setVisible(False)

        splash_write(QtGui.QApplication.translate("Splash", "Loading icons"))
        self.ICONS = CompleteIcons()

        splash_write(QtGui.QApplication.translate("Splash", "Setting theme"))
        self.build_menutoolbar()
        self.set_icon_theme()
        self.reload_toolbar_icons()

        splash_write(QtGui.QApplication.translate("Splash", "Linking paths for libraries and compilers"))
        PinguinoConfig.update_pinguino_paths(self.configIDE, self.pinguinoAPI)
        PinguinoConfig.update_pinguino_extra_options(self.configIDE, self.pinguinoAPI)
        splash_write(QtGui.QApplication.translate("Splash", "Searching user libraries"))
        PinguinoConfig.update_user_libs(self.pinguinoAPI)

        self.setWindowTitle(os.getenv("PINGUINO_FULLNAME"))
        PythonShell.__init__(self)
        Log.__init__(self)
        #Help.__init__(self)

        splash_write(QtGui.QApplication.translate("Splash", "Opening last files"))
        self.ide_open_last_files()

        splash_write(QtGui.QApplication.translate("Splash", "Starting widgets features"))
        self.init_widgets()
        splash_write(QtGui.QApplication.translate("Splash", "Building status bar"))
        self.build_statusbar()
        # splash_write(QtGui.QApplication.translate("Splash", "Building tabs"))
        # self.build_tabs()

        splash_write(QtGui.QApplication.translate("Splash", "Overwriting stylesheets"))
        self.set_styleSheet()

        #timer events
        splash_write(QtGui.QApplication.translate("Splash", "Starting timers"))
        self.timer_update_functions()
        self.timer_update_directives()
        self.timer_update_variables()
        self.timer_update_autocompleter()
        self.timer_check_changes()
        self.timer_backup_file()
        self.timer_update_assiatant()

        splash_write(QtGui.QApplication.translate("Splash", "Loading examples"))
        self .change_dir_files(0)  #Examples
        # self.update_path_files([os.path.join(os.getenv("PINGUINO_USER_PATH"), "examples"),
                                # os.path.join(os.getenv("PINGUINO_USER_PATH"), "graphical_examples")])
        # self.__update_graphical_path_files__(os.path.join(os.getenv("PINGUINO_USER_PATH"), "graphical_examples"))

        splash_write(QtGui.QApplication.translate("Splash", "Loading boards configuration"))
        #self.set_board() #called in self.get_status_board()
        self.status_info.setText(self.get_status_board())

        splash_write(QtGui.QApplication.translate("Splash", "Connecting events"))
        self.connect_events()
        splash_write(QtGui.QApplication.translate("Splash", "Loading configuration"))
        self.load_main_config()
        self.update_project_status(None)
        splash_write(QtGui.QApplication.translate("Splash", "Checking configuration files"))
        self.ide_check_files()

        splash_write(QtGui.QApplication.translate("Splash", "Loading autocompleter"))
        self.PinguinoAutoCompleter = PinguinoAutoCompleter()

        os_name = os.getenv("PINGUINO_OS_NAME")
        if os_name == "windows":
            os.environ["PATH"] = os.environ["PATH"] + ";" + self.configIDE.get_path("sdcc_bin")

        elif os_name == "linux":
            os.environ["LD_LIBRARY_PATH"]="/usr/lib32:/usr/lib:/usr/lib64"

        elif os_name == "macosx":
            #FIXME_OSX
            os.environ["LD_LIBRARY_PATH"]="/usr/lib32:/usr/lib:/usr/lib64"  #Confirm this for macosx

        splash_write(QtGui.QApplication.translate("Splash", "Welcome to {PINGUINO_NAME} {PINGUINO_VERSION}".format(**os.environ)))

        self.enable_debugger()
        logging.info("Welcome to {PINGUINO_FULLNAME}".format(**os.environ))

        self.setCorner(QtCore.Qt.BottomRightCorner, QtCore.Qt.RightDockWidgetArea)
        self.setCorner(QtCore.Qt.BottomLeftCorner, QtCore.Qt.LeftDockWidgetArea)
        self.set_sise_hint()

        PinguinoEvents.__init__(self)
        self.__startapp__()

    #----------------------------------------------------------------------
    def set_sise_hint(self):
        """"""
        elements = ["dockWidget_bottom", "dockWidget_right", "plainTextEdit_log", "plainTextEdit_output", "tabWidget_bottom",
                    "tabWidget_tools", "comboBox_files", "treeWidget_explorer", "lineEdit_blocks_search",
                    "label_search_block", "pushButton_openproject", "pushButton_newproject", "treeWidget_projects",
                    "tableWidget_variables", "lineEdit_search", "lineEdit_replace", "checkBox_case_sensitive", "label_search",
                    "label_replace", "pushButton_search_previous", "pushButton_search_next", "pushButton_replace",
                    "pushButton_replace_all", "label_replace_info", "plainTextEdit_stdout"]

        for element in elements:
            getattr(self.main, element).minimumSizeHint = lambda :QtCore.QSize(0, 0)


    #----------------------------------------------------------------------
    def init_widgets(self):

        self.main.tabWidget_tools.setCurrentWidget(self.main.Files)
        self.main.tabWidget_bottom.setCurrentWidget(self.main.Log)

        self.main.dockWidget_right.setTitleBarWidget(QtGui.QWidget())
        self.main.dockWidget_bottom.setTitleBarWidget(QtGui.QWidget())


    #----------------------------------------------------------------------
    def set_styleSheet(self):
        #self.load_fonts()

        self.PinguinoPallete = BackgroundPallete()
        self.PinguinoPallete.save_palette(self.main.centralwidget.parent())
        self.switch_color_theme(self.configIDE.config("Main", "color_theme", False))

        bg_color = self.configIDE.config("Styles", "background_color", "#FFFFFF")
        alternate_bg_color = self.configIDE.config("Styles", "alternate_background_color", "#DDE8FF")
        selection_color = self.configIDE.config("Styles", "selection_color", "#FFFFFF")
        selection_bg_color = self.configIDE.config("Styles", "selection_foreground_color", "#57AAFF")

        self.main.tableWidget_functions.setStyleSheet("""
        QTableWidget {{
            background-color: {};
            alternate-background-color: {};
        }}
        """.format(bg_color, alternate_bg_color))

        self.main.tableWidget_directives.setStyleSheet("""
        QTableWidget {{
            background-color: {};
            alternate-background-color: {};
        }}
        """.format(bg_color, alternate_bg_color))

        self.main.tableWidget_variables.setStyleSheet("""
        QTableWidget {{
            background-color: {};
            alternate-background-color: {};
        }}
        """.format(bg_color, alternate_bg_color))

        self.main.statusBar.setStyleSheet("""
        font-family: inherit;
        font-weight: normal;
        selection-color: {};
        selection-background-color: {};
        """.format(selection_color, selection_bg_color))

        #Python shell CSS styles
        self.main.plainTextEdit_output.setStyleSheet("""
        QPlainTextEdit {
            background-color: #000;
            color: #FFFFFF;
            font-family: mono;
            font-weight: normal;
            font-size: 10pt;
        }
        """)

        #Log CSS styles
        self.main.plainTextEdit_log.setStyleSheet("""
        QPlainTextEdit {
            background-color: #000;
            color: #FFFFFF;
            font-family: mono;
            font-weight: normal;
            font-size: 10pt;
        }
        """)

        #StdOut CSS styles
        self.main.plainTextEdit_stdout.setStyleSheet("""
        QPlainTextEdit {
            background-color: #FFFFFF;
            font-family: mono;
            font-weight: normal;
            font-size: 10pt;
        }
        """)


if os.getenv("PINGUINO_MODE") == "NORMAL":
    for name, fn in inspect.getmembers(PinguinoIDE):
        # if isinstance(fn, types.UnboundMethodType):
        if isinstance(fn, (types.MethodType, types.FunctionType)):
            setattr(PinguinoIDE, name, Decorator.debug_method()(fn))


#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os

from PySide import QtGui, QtCore

from ...frames.about import Ui_About

import resources_rc

########################################################################
class About(QtGui.QDialog):

    def __init__(self):
        super(About, self).__init__()
        self.setWindowFlags(QtCore.Qt.WindowCloseButtonHint |
                            QtCore.Qt.WindowSystemMenuHint |
                            QtCore.Qt.WindowStaysOnTopHint |
                            QtCore.Qt.Tool)

        self.about = Ui_About()
        self.about.setupUi(self)

        self.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+self.windowTitle())

        self.about.label_name.setText(os.getenv("PINGUINO_FULLNAME"))

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/logo/art/windowIcon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)

        self.connect(self.about.pushButton_credits, QtCore.SIGNAL("clicked()"), lambda :self.about.stackedWidget.setCurrentIndex(1))
        self.connect(self.about.pushButton_license, QtCore.SIGNAL("clicked()"), lambda :self.about.stackedWidget.setCurrentIndex(2))
        self.connect(self.about.pushButton_about, QtCore.SIGNAL("clicked()"), lambda :self.about.stackedWidget.setCurrentIndex(0))
        self.connect(self.about.pushButton_close, QtCore.SIGNAL("clicked()"), self.close)
        self.connect(self.about.pushButton_close_2, QtCore.SIGNAL("clicked()"), self.close)
        self.connect(self.about.pushButton_close_3, QtCore.SIGNAL("clicked()"), self.close)

        self.about.stackedWidget.setCurrentIndex(0)
        self.about.tabWidget.setCurrentIndex(0)

        self.setStyleSheet("""
        font-family: inherit;
        font-weight: normal;

        """)

        self.center_on_screen()

    #----------------------------------------------------------------------
    def center_on_screen(self):

        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)



#! /usr/bin/python
#-*- coding: utf-8 -*-

import datetime
today = datetime.datetime.now()

Autocompleter = {}
Autocompleter["directive"] = ["define", "include", "ifndef", "endif", "undef", "if", "elif", "else", "error", "warning"]

#from const.h
const = [ "PI", "HALF_PI", "TWO_PI", "DEG_TO_RAD", "RAD_TO_DEG", "NULL", "ON", "OFF", "FALSE", "TRUE", "True", "False", "false", "true",
          "INPUT", "OUTPUT", "HIGH",
          "LOW", "AND", "OR",
          "BYTE", "BIN", "OCT", "DEC", "HEX", "FLOAT",
          "LED1", "LED2", "LED3", "LED4", "WHITELED", "GREENLED", "USERLED", "YELLOWLED", "REDLED", "PROGBUTTON", "USERBUTTON",
          "RTCC", "PMCS1", "PMRD", "PMWR", "PMA1",


          #Others, not in cons.h
          "FOSC", "MIPS",
          "PORTA", "PORTB", "PORTC", "PORTD", "PORTE", "PORTF", "PORTG",

          "void", "const", "BOOL", "char", "unsigned", "short", "int", "long", "float", "double", "byte", "word",
          "u8", "s8", "u16", "s16", "u32", "s32", "u64", "s64", "PUBLIC",

          "struct", "union", "typedef", "enum", "register", "static", "extern", "volatile",
          "loop", "setup", "INT_MILLISEC", "INT_MICROSEC", "INT_FALLING_EDGE", "interrupt",


          #C syntax
          "if", "switch", "for", "while", "do", "continue", "break", "else", "return", "case", "default",
]


const += ["P1_{}".format(p) for p in range(1, 17)]
const += ["P2_{}".format(p) for p in range(1, 17)]
const += ["D{}".format(p) for p in range(14)]
const += ["A{}".format(p) for p in range(8)]
const += ["PWM{}".format(p) for p in range(5)]
const += ["PWD{}".format(p) for p in range(8)]

Autocompleter["reserved"] = const

Snippet = {}
#Snippet[name {snippet}]=["to insert, with cursor mark: [!]"] ##[!] is the cursor position

Snippet["void {snippet}"] = "void [!]() {\n\n}"
Snippet["while( ) {snippet}"] = "while ([!]) {\n\n}"
Snippet["do {...} while  {snippet}"] = "do {\n\t[!]\n} while();"
Snippet["for {snippet}"] = "for ([!]) {\n\n}"
Snippet["for (i=0;...{snippet}"] = "for (i=[!]; i<; i++) {\n\n}"
Snippet["if...else {snippet}"] = "if ([!]) {\n\n\t}\nelse {\n\n}"
Snippet["if...else if...else {snippet}"] = "if ([!]) {\n\n}\nelse if () {\n\n}\nelse {\n\n}"
Snippet["switch( ) {snippet}"] = "switch ([!]) {\n\tcase:\n\n\t\tbreak;\n\n\tdefault:\n\n\t\tbreak;}"
Snippet["case {snippet}"] = "case [!]: \n\n\tbreak;"

Snippet["struct {snippet}"] = "struct {\n\t[!]\n};"
Snippet["typedef ... struct {snippet}"] = "typedef struct {\n\t[!]\n}TYPE;"
Snippet["union {snippet}"] = "union [!] {\n\t\n};"
Snippet["typedef ... union {snippet}"] = "typedef union {\n\t[!]\n}TYPE;"

Snippet["Bare minimum {snippet}"] = "void setup() {\n\t// put your setup code here, to run once:\n\n\t[!]\n}\n\nvoid loop() {\n\t// put your main code here, to run repeatedly:\n\n}\n"


Snippet["file {snippet}"] = """/*-----------------------------------------------------
{}: [!] --<>
{}: {}
{}:

-----------------------------------------------------*/
""".format("Author", "Date", today.strftime("%Y-%m-%d"), "Description")

for key in Snippet.keys():
    Snippet[key] = Snippet[key].replace("\t", " "*4)


Helpers = {
    "pinMode": "pinMode({{USERLED, OUTPUT}});",
    "digitalRead": "digitalRead({{PIN}});",
    "analogRead": "analogRead({{PIN}});",
    "digitalWrite": "digitalWrite({{USERLED, HIGH}});",
    "analogWrite": "digitalWrite({{PIN, 256}});",
    "delay": "delay({{MILLISECONDS}});",

    "toggle": "toggle({{USERLED}});",
    "random": "random({{1, 10}});",

    "Serial.begin": "Serial.begin({{9600}});",
    "Serial.printNumber": "Serial.printNumber({{256, DEC}});",

    "lcd.pins": "lcd.pins({{0, 1, 2, 3, 4, 5, 0, 0, 0, 0}}); // RS, E, D4 ~ D8",
    "lcd.begin": "lcd.begin({{16, 2}});"
}


#! /usr/bin/python2
# -*- coding: iso-8859-15 -*-

"""---------------------------------------------------------------------
    Pinguino Boards List

    author:			Regis Blanchot
    contact:		rblanchot@gmail.com
    first release:	2011-10-23
    last release:	2015-09-08
    --------------------------------------------------------------------
    2015-09-08      RB - added Pinguino 16F1459
    --------------------------------------------------------------------
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
---------------------------------------------------------------------"""

# vendors id
MICROCHIP_ID = 0x04D8

# products id (Pinguino's Microchip sub-license)
P8_ID  = 0xFEAA
P32_ID = 0x003C

# RB 17/09/2013
"""
class PinguinoNoBoot(object):
    name = 'Pinguino (no Bootloader)'
    arch = 8
    bldr = 'noboot'
    proc = ''
    board = ''
    vendor = ''
    product = ''
    memstart = 0x0000
    memend = 0xFFF8
    shortarg = '-b'
    longarg = '--noboot'
"""

"""---------------------------------------------------------------------
    8-bit (16F) Boards
---------------------------------------------------------------------"""

class Pinguino1459(object):
    name     = 'Pinguino 1459'
    arch     = 8
    bldr     = 'boot4'
    proc     = '16f1459'
    board    = 'PINGUINO1459'
    family   = '16F'
    vendor   = MICROCHIP_ID
    product  = P8_ID
    memstart = 0x0500       # bootloader offset
    memend   = 0x2000
    shortarg = '-p1459'
    longarg  = '--pinguino1459'

class Curiosity(object):
    name     = 'Curiosity 1708'
    arch     = 8
    bldr     = 'noboot'
    proc     = '16f1708'
    board    = 'CURIOSITY1708'
    family   = '16F'
    vendor   = MICROCHIP_ID
    product  = P8_ID
    memstart = 0x0000
    memend   = 0x1000
    shortarg = '-c1708'
    longarg  = '--curiosity1708'

"""---------------------------------------------------------------------
    8-bit (18F) Boards
---------------------------------------------------------------------"""

class Pinguino13k50(object):
    name = 'Pinguino 13K50'
    arch = 8
    bldr = 'boot4'
    proc = '18f13k50'
    board = 'PINGUINO13K50'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x2000
    shortarg = '-p13k50'
    longarg = '--pinguino13k50'

class Pinguino14k50(object):
    name = 'Pinguino 14K50'
    arch = 8
    bldr = 'boot4'
    proc = '18f14k50'
    board = 'PINGUINO14K50'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x4000
    shortarg = '-p14k50'
    longarg = '--pinguino14k50'

class Pinguino2455(object):
    name = 'Pinguino 2455'
    arch = 8
    bldr = 'boot4'
    proc = '18f2455'
    board = 'PINGUINO2455'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x6000
    shortarg = '-p2455'
    longarg = '--pinguino2455'

class Pinguino2550(object):
    name = 'Pinguino 2550'
    arch = 8
    bldr = 'boot4'
    proc = '18f2550'
    board = 'PINGUINO2550'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x8000
    shortarg = '-p2550'
    longarg = '--pinguino2550'

class FreeJALduino(object):
    name = 'FreeJALduino'
    arch = 8
    bldr = 'boot4'
    proc = '18f2550'
    board = 'FREEJALDUINO'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00
    memend   = 0x8000
    config = 0x300000
    shortarg = '-J'
    longarg = '--freejalduino'

class Pinguino25k50(object):
    name = 'Pinguino 25K50'
    arch = 8
    bldr = 'boot4'
    proc = '18f25k50'
    board = 'PINGUINO25K50'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x8000
    shortarg = '-p25k50'
    longarg = '--pinguino25k50'

class Pinguino26J50(object):
    name = 'Pinguino 26J50'
    arch = 8
    bldr = 'boot4'
    proc = '18f26j50'
    board = 'PINGUINO26J50'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x10000
    shortarg = '-p26j50'
    longarg = '--pinguino26j50'

class Pinguino27J53(object):
    name = 'Pinguino 27J53'
    arch = 8
    bldr = 'boot4'
    proc = '18f27j53'
    board = 'PINGUINO27J53'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x20000
    shortarg = '-p27j53'
    longarg = '--pinguino27j53'

class Pinguino4455(object):
    name = 'Pinguino 4455'
    arch = 8
    bldr = 'boot4'
    proc = '18f4455'
    board = 'PINGUINO4455'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x6000
    shortarg = '-p4455'
    longarg = '--pinguino4455'

class Pinguino4550(object):
    name = 'Pinguino 4550'
    arch = 8
    bldr = 'boot4'
    proc = '18f4550'
    board = 'PINGUINO4550'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x8000
    shortarg = '-p4550'
    longarg = '--pinguino4550'

class PICuno_Equo(object):
    name = 'PICuno Equo'
    arch = 8
    bldr = 'boot4'
    proc = '18f4550'
    board = 'PICUNO_EQUO'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00
    memend   = 0x8000
    config = 0x300000
    shortarg = '-eq'
    longarg = '--picunoequo'

class Pinguino45k50(object):
    name = 'Pinguino 45K50'
    arch = 8
    bldr = 'boot4'
    proc = '18f45k50'
    board = 'PINGUINO45K50'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x8000
    shortarg = '-p45k50'
    longarg = '--pinguino45k50'

class CHRP3(object):
    name = 'CHRP3.0'
    arch = 8
    bldr = 'boot4'
    proc = '18f25k50'
    board = 'CHRP3'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x8000
    shortarg = '-c'
    longarg = '--chrp3'

class Pinguino46J50(object):
    name = 'Pinguino 46J50'
    arch = 8
    bldr = 'boot4'
    proc = '18f46j50'
    board = 'PINGUINO46J50'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x10000
    shortarg = '-p46j50'
    longarg = '--pinguino46j50'

class Pinguino47J53A(object):
    name = 'Pinguino 47J53A'
    arch = 8
    bldr = 'boot4'
    proc = '18f47j53'
    board = 'PINGUINO47J53A'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x20000
    shortarg = '-p47j53a'
    longarg = '--pinguino47j53a'

class Pinguino47J53B(object):
    name = 'Pinguino Torda'
    arch = 8
    bldr = 'boot4'
    proc = '18f47j53'
    board = 'PINGUINO47J53B'
    family = '18F'
    vendor = MICROCHIP_ID
    product = P8_ID
    memstart = 0x0C00       # bootloader offset
    memend   = 0x20000
    shortarg = '-pTorda'
    longarg = '--pinguinoTorda'

"""---------------------------------------------------------------------
    32-bit (MX) Boards
---------------------------------------------------------------------"""

class Pinguino32MX220(object):
    name = 'Pinguino 32MX220'
    arch = 32
    bldr = 'microchip'
    proc = '32MX220F032B'
    board = 'PINGUINO32MX220'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D003000           # _ebase_address (exception_mem)
    memstart = 0x9D004010           # kseg0_program_mem
    memend   = 0x9D008000
    shortarg = '-p220'
    longarg = '--pinguino32mx220'

class Pinguino32MX250(object):
    name = 'Pinguino 32MX250'
    arch = 32
    bldr = 'microchip'
    proc = '32MX250F128B'
    board = 'PINGUINO32MX250'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D003000           # _ebase_address (exception_mem)
    memstart = 0x9D004010           # kseg0_program_mem
    memend   = 0x9D020000
    shortarg = '-p250'
    longarg = '--pinguino32mx250'

class Pinguino32MX270(object):
    name = 'Pinguino 32MX270'
    arch = 32
    bldr = 'microchip'
    proc = '32MX270F256B'
    board = 'PINGUINO32MX270'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D002000           # _ebase_address (exception_mem)
    memstart = 0x9D003010           # kseg0_program_mem
    memend   = 0x9D040000
    shortarg = '-p270'
    longarg = '--pinguino32mx270'

class PIC32_Pinguino_220(object):
    name = 'PIC32 Pinguino MX220'
    arch = 32
    bldr = 'microchip'
    proc = '32MX220F032D'
    board = 'PIC32_PINGUINO_220'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D003000           # _ebase_address (exception_mem)
    memstart = 0x9D004010           # kseg0_program_mem
    memend   = 0x9D008000
    shortarg = '-m'
    longarg = '--olimex220'

class PIC32_Pinguino(object):
    name = 'PIC32 Pinguino'
    arch = 32
    bldr = 'microchip'
    proc = '32MX440F256H'
    board = 'PIC32_PINGUINO'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D040000
    shortarg = '-o'
    longarg = '--olimex440'

class PIC32_Pinguino_OTG(object):
    name = 'PIC32 Pinguino OTG'
    arch = 32
    bldr = 'microchip'
    proc = '32MX440F256H'
    board = 'PIC32_PINGUINO_OTG'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D040000
    shortarg = '-O'
    longarg = '--olimex440OTG'

class PIC32_Pinguino_Micro(object):
    name = 'PIC32 Pinguino Micro'
    arch = 32
    bldr = 'microchip'
    proc = '32MX440F256H'
    board = 'PIC32_PINGUINO_MICRO'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D040000
    shortarg = '-M'
    longarg = '--olimex440Micro'

class PIC32_Pinguino_T795(object):
    name = 'PIC32 Pinguino T795'
    arch = 32
    bldr = 'microchip'
    proc = '32MX795F512H'
    board = 'PIC32_PINGUINO_T795'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D080000
    shortarg = '-T'
    longarg = '--olimexT795'

class Emperor_460(object):
    name = 'Emperor 460'
    arch = 32
    bldr = 'microchip'
    proc = '32MX460F512L'
    board = 'EMPEROR460'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D080000
    shortarg = '-e'
    longarg = '--emperor460'

class Emperor_795(object):
    name = 'Emperor 795'
    arch = 32
    bldr = 'microchip'
    proc = '32MX795F512L'
    board = 'EMPEROR795'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D080000
    shortarg = '-E'
    longarg = '--emperor795'

class UBW32_460(object):
    name = 'UBW32 460'
    arch = 32
    bldr = 'microchip'
    proc = '32MX460F512L'
    board = 'UBW32_460'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D080000
    shortarg = '-u'
    longarg = '--ubw460'

class UBW32_795(object):
    name = 'UBW32 795'
    arch = 32
    bldr = 'microchip'
    proc = '32MX795F512L'
    board = 'UBW32_795'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D080000
    shortarg = '-U'
    longarg = '--ubw795'

"""---------------------------------------------------------------------
    32-bit (MZ) Boards
---------------------------------------------------------------------"""

class PIC32MZSTARTERKIT(object):
    name = 'PIC32MZ Starter Kit'
    arch = 32
    bldr = 'microchip'
    proc = '32MZ2048ECH144'
    board = 'PIC32MZSTARTERKIT'
    vendor = MICROCHIP_ID
    product = P32_ID
    ivtstart = 0x9D005000           # _ebase_address (exception_mem)
    memstart = 0x9D006010           # kseg0_program_mem
    memend   = 0x9D080000
    shortarg = '-Z2048'
    longarg = '--pic32zsk'

"""---------------------------------------------------------------------
    Board list
---------------------------------------------------------------------"""

    # RB 17/09/2013
    #PinguinoNoBoot,
boardlist = [
    Pinguino1459,  Curiosity,
    Pinguino13k50, Pinguino14k50,
    Pinguino2455,  Pinguino4455,
    Pinguino2550,  Pinguino4550,
    FreeJALduino,  PICuno_Equo, CHRP3,
    Pinguino25k50, Pinguino45k50,
    Pinguino26J50, Pinguino46J50,
    Pinguino27J53, Pinguino47J53A, Pinguino47J53B,
    Pinguino32MX220, Pinguino32MX250, Pinguino32MX270,
    PIC32_Pinguino, PIC32_Pinguino_OTG,
    PIC32_Pinguino_Micro, PIC32_Pinguino_T795,
    PIC32_Pinguino_220,
    Emperor_460, Emperor_795,
    UBW32_460, UBW32_795,
    PIC32MZSTARTERKIT
]



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from . import constructor as Block
from .constructor import Kit

Kit = Kit()

########################################################################
class Stepper(object):

    Stepper_setSpeed = lambda n:Kit.addFunct("Stepper", "Stepper.setSpeed", 1, fill=["spin-int"], str_help="in revolutions per minute")
    Stepper_setMicrostep = lambda n:Kit.addFunct("Stepper", "Stepper.setMicrostep", 1, fill=["spin-int"])
    Stepper_step = lambda n:Kit.addFunct("Stepper", "Stepper.step", 1, fill=["spin-int"])


    #----------------------------------------------------------------------
    def Stepper_init(self):

        pm = Block.Linear("Stepper")
        pm.addLabel("Stepper.init")
        pm.addSyntax("(")
        pm.addArg("STEPS")
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addDecorator("PINS=")
        for i in range(3):
            pm.addSpace()
            pm.addSyntax(",")
        pm.addSpace()
        pm.addSyntax(")")
        return pm.getBlock()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/gide/bloques/nested_in_code_next/nested_second.ui'
#
# Created: Wed Mar 16 13:19:38 2016
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(109, 89)
        Form.setWindowTitle("")
        self.gridLayout = QtGui.QGridLayout(Form)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.frame_5 = QtGui.QFrame(Form)
        self.frame_5.setMinimumSize(QtCore.QSize(42, 24))
        self.frame_5.setMaximumSize(QtCore.QSize(42, 24))
        self.frame_5.setObjectName("frame_5")
        self.horizontalLayout_5 = QtGui.QHBoxLayout(self.frame_5)
        self.horizontalLayout_5.setSpacing(0)
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtGui.QLabel(self.frame_5)
        self.label_4.setText("")
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.horizontalLayout_5.addLayout(self.horizontalLayout_4)
        self.gridLayout.addWidget(self.frame_5, 2, 0, 1, 2)
        self.frame_3 = QtGui.QFrame(Form)
        self.frame_3.setMinimumSize(QtCore.QSize(6, 46))
        self.frame_3.setMaximumSize(QtCore.QSize(6, 46))
        self.frame_3.setObjectName("frame_3")
        self.horizontalLayout_9 = QtGui.QHBoxLayout(self.frame_3)
        self.horizontalLayout_9.setSpacing(0)
        self.horizontalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.horizontalLayout_8 = QtGui.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_3 = QtGui.QLabel(self.frame_3)
        self.label_3.setText("")
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_8.addWidget(self.label_3)
        self.horizontalLayout_9.addLayout(self.horizontalLayout_8)
        self.gridLayout.addWidget(self.frame_3, 0, 2, 1, 1)
        self.frame = QtGui.QFrame(Form)
        self.frame.setMinimumSize(QtCore.QSize(38, 46))
        self.frame.setMaximumSize(QtCore.QSize(38, 46))
        self.frame.setObjectName("frame")
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.frame)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtGui.QLabel(self.frame)
        self.label.setText("")
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_2)
        self.gridLayout.addWidget(self.frame, 0, 0, 1, 1)
        self.frame_4 = QtGui.QFrame(Form)
        self.frame_4.setMinimumSize(QtCore.QSize(11, 0))
        self.frame_4.setMaximumSize(QtCore.QSize(11, 16777215))
        self.frame_4.setObjectName("frame_4")
        self.horizontalLayout_7 = QtGui.QHBoxLayout(self.frame_4)
        self.horizontalLayout_7.setSpacing(0)
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_5 = QtGui.QLabel(self.frame_4)
        self.label_5.setText("")
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_6.addWidget(self.label_5)
        self.horizontalLayout_7.addLayout(self.horizontalLayout_6)
        self.gridLayout.addWidget(self.frame_4, 1, 0, 1, 1)
        self.frame_2 = QtGui.QFrame(Form)
        self.frame_2.setMinimumSize(QtCore.QSize(0, 46))
        self.frame_2.setMaximumSize(QtCore.QSize(16777215, 46))
        self.frame_2.setObjectName("frame_2")
        self.gridLayout_2 = QtGui.QGridLayout(self.frame_2)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 8)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtGui.QLabel(self.frame_2)
        self.label_2.setText("")
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.frame_2, 0, 1, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        pass




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'submit_bug.ui'
#
# Created: Mon Feb 20 23:09:17 2017
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_SubmitBug(object):
    def setupUi(self, SubmitBug):
        SubmitBug.setObjectName("SubmitBug")
        SubmitBug.resize(652, 594)
        self.gridLayout = QtGui.QGridLayout(SubmitBug)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtGui.QLabel(SubmitBug)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_4 = QtGui.QLabel(SubmitBug)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.lineEdit_username = QtGui.QLineEdit(SubmitBug)
        self.lineEdit_username.setObjectName("lineEdit_username")
        self.horizontalLayout_2.addWidget(self.lineEdit_username)
        self.label_5 = QtGui.QLabel(SubmitBug)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.lineEdit_password = QtGui.QLineEdit(SubmitBug)
        self.lineEdit_password.setEchoMode(QtGui.QLineEdit.Password)
        self.lineEdit_password.setObjectName("lineEdit_password")
        self.horizontalLayout_2.addWidget(self.lineEdit_password)
        self.gridLayout.addLayout(self.horizontalLayout_2, 2, 0, 1, 1)
        self.verticalLayout_3 = QtGui.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_6 = QtGui.QLabel(SubmitBug)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_3.addWidget(self.label_6)
        self.comboBox_repo = QtGui.QComboBox(SubmitBug)
        self.comboBox_repo.setObjectName("comboBox_repo")
        self.comboBox_repo.addItem("")
        self.comboBox_repo.addItem("")
        self.comboBox_repo.addItem("")
        self.comboBox_repo.addItem("")
        self.verticalLayout_3.addWidget(self.comboBox_repo)
        self.gridLayout.addLayout(self.verticalLayout_3, 4, 0, 1, 1)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_2 = QtGui.QLabel(SubmitBug)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.lineEdit_summary = QtGui.QLineEdit(SubmitBug)
        self.lineEdit_summary.setObjectName("lineEdit_summary")
        self.verticalLayout.addWidget(self.lineEdit_summary)
        self.gridLayout.addLayout(self.verticalLayout, 5, 0, 1, 1)
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_3 = QtGui.QLabel(SubmitBug)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.plainTextEdit_details = QtGui.QPlainTextEdit(SubmitBug)
        self.plainTextEdit_details.setObjectName("plainTextEdit_details")
        self.verticalLayout_2.addWidget(self.plainTextEdit_details)
        self.gridLayout.addLayout(self.verticalLayout_2, 6, 0, 1, 1)
        self.checkBox_show_this_dialog = QtGui.QCheckBox(SubmitBug)
        self.checkBox_show_this_dialog.setChecked(True)
        self.checkBox_show_this_dialog.setObjectName("checkBox_show_this_dialog")
        self.gridLayout.addWidget(self.checkBox_show_this_dialog, 7, 0, 1, 1)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pushButton_cancel = QtGui.QPushButton(SubmitBug)
        self.pushButton_cancel.setObjectName("pushButton_cancel")
        self.horizontalLayout.addWidget(self.pushButton_cancel)
        self.pushButton_submit = QtGui.QPushButton(SubmitBug)
        self.pushButton_submit.setDefault(True)
        self.pushButton_submit.setFlat(False)
        self.pushButton_submit.setObjectName("pushButton_submit")
        self.horizontalLayout.addWidget(self.pushButton_submit)
        self.gridLayout.addLayout(self.horizontalLayout, 8, 0, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem1, 3, 0, 1, 1)
        spacerItem2 = QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem2, 1, 0, 1, 1)

        self.retranslateUi(SubmitBug)
        QtCore.QMetaObject.connectSlotsByName(SubmitBug)

    def retranslateUi(self, SubmitBug):
        SubmitBug.setWindowTitle(QtGui.QApplication.translate("SubmitBug", "Submit Bug", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("SubmitBug", "<html><head/><body>\n"
"<p>Please provide as much information as possible about wath caused the bug that you are reporting.</p>\n"
"<p>This feature could recollect extra information about system, like OS, architecture, PYTHONPATH,<br>username, etc.</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("SubmitBug", "GitHub username:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("SubmitBug", "GitHub password", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("SubmitBug", "Repository:", None, QtGui.QApplication.UnicodeUTF8))
        self.comboBox_repo.setItemText(0, QtGui.QApplication.translate("SubmitBug", "pinguino-ide", None, QtGui.QApplication.UnicodeUTF8))
        self.comboBox_repo.setItemText(1, QtGui.QApplication.translate("SubmitBug", "pinguino-bootloaders", None, QtGui.QApplication.UnicodeUTF8))
        self.comboBox_repo.setItemText(2, QtGui.QApplication.translate("SubmitBug", "pinguino-libraries", None, QtGui.QApplication.UnicodeUTF8))
        self.comboBox_repo.setItemText(3, QtGui.QApplication.translate("SubmitBug", "pinguino-compilers", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("SubmitBug", "Summary:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("SubmitBug", "Details:", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBox_show_this_dialog.setText(QtGui.QApplication.translate("SubmitBug", "Show this dialog on unexpected errors.", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_cancel.setText(QtGui.QApplication.translate("SubmitBug", "Cancel", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_submit.setText(QtGui.QApplication.translate("SubmitBug", "Submit Now", None, QtGui.QApplication.UnicodeUTF8))




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import re

data_types = ["int", "float", "char", "double", "u8", "u16", "u32", "u64", "BOOL", "byte", "word", "void", "short", "long"]  # Yes I know, void is NOT a data type.
preprocessor_commands = ["define", "include", "error", "undef", "if", "else", "if", "elif", "ifdef", "ifndef", "line", "pragma"]  #endif

#----------------------------------------------------------------------
def remove_comments(textinput, ignore_spaces=False):

    if type(textinput) == list:
        text = "".join(textinput)
    else:
        text = textinput

    def replacer(match):
        s = match.group(0)

        if s.startswith('/'):
            if not ignore_spaces:
                # return " "*(len(s)-s.count("\n")) + "\n" * (s.count("\n"))
                return "" + "\n" * (s.count("\n"))
            else:
                return ""
        else:
            return s

    pattern = re.compile(r'//.*?$|/\*.*?\*/|\'(?:\\.|[^\\\'])*\'|"(?:\\.|[^\\"])*"', re.DOTALL|re.MULTILINE)
    textout = re.sub(pattern, replacer, text)

    if type(textinput) == list:
        textout = textout.split("\n")
        textout = list(map(lambda x:x+"\n", textout))

    return textout



########################################################################
class PinguinoParser(object):
    """"""
    #----------------------------------------------------------------------
    def __init__(self, files):
        """"""
        self.files = files


    #----------------------------------------------------------------------
    def get_variables(self):
        """"""
        regex_variables = "[\s]*(volatile|register|static|extern)*[\s]*(unsigned|signed)*[\s]*(" + "|".join(data_types) + ")+[\s]*( \* )*[\s]*([ \w\[\]=,.{}\"']*);"
        # regex_variables = "[\s]*(volatile|register|static|extern)*[\s]*(unsigned|signed)*[\s]*(short|long)*[\s]*(" + "|".join(data_types) + ")[\s]*+([*])*([ \w\[\]=,.{}\"'\*]*);"

        variables = []

        # self.signal_set_variables.emit(variables)

        # # if files is None:
        # files = self.ide.get_files_to_explore()

        for file_ in self.files:

            filename = file_["filename"]
            content = file_["content"]

            content = remove_comments(content)
            content = content.split("\n")

            for line in range(len(content)):
                match = re.match(regex_variables, content[line])
                if match:
                    args = match.groups()[4]
                    l = []
                    index = 0
                    while index < len(args):
                        if args[index] == "{":
                            start = index
                            bracket = 1

                            while bracket != 0:
                                index += 1
                                if args[index] == "{": bracket += 1
                                if args[index] == "}": bracket -= 1
                            end = index

                            l.append(args[start:end+1])

                        else: index += 1

                    for j in l: args = args.replace(j, "", 1)
                    args = args.split(",")
                    args = map(lambda a:a[:a.find("=")] if "=" in a else a, args)

                    type_ = " ".join([("" if not match.groups()[3] else "pointer to "),
                                    ("" if not match.groups()[0] else match.groups()[0]),
                                    ("" if not match.groups()[1] else match.groups()[1]),
                                    (match.groups()[2]),
                                    # match.groups()[3]
                                    ])

                    while type_.startswith(" "): type_ = type_[1:]

                    for arg in args:
                        this_variable = {}
                        this_variable["type"] = type_
                        if re.match("(.*)(\[.*\])", arg):
                            this_variable["name"] = re.match("(.*)(\[.*\])", arg).groups()[0]
                        else:
                            this_variable["name"] = arg
                        #this_variable["value"] = match.groups()[6]
                        this_variable["line"] = line + 1
                        this_variable["filename"] = filename
                        variables.append(this_variable)


        return variables


    #----------------------------------------------------------------------
    def get_directives(self):

        regex_directive = "[\s]*#(" + "|".join(preprocessor_commands)+ ")[\s]+<?[\s]*([\w.]*)[\s]*>?[\s]*([\S]*)"

        directives = []

        # files = self.ide.get_files_to_explore()

        for file_ in self.files:

            filename = file_["filename"]
            content = file_["content"]

            content = remove_comments(content)
            content = content.split("\n")

            for line in range(len(content)):
                match = re.match(regex_directive, content[line])
                this_directive = {}
                if match:
                    this_directive["type"] = "#" + match.groups()[0]
                    this_directive["name"] = match.groups()[1]
                    this_directive["value"] = match.groups()[2]
                    this_directive["line"] = line + 1
                    this_directive["filename"] = filename
                    directives.append(this_directive)

        return directives


    #----------------------------------------------------------------------
    def get_functions(self):

        regex_function = "[\s]*(unsigned|signed|static|long|PUBLIC)*[\s]*(" + "|".join(data_types) + ")[\s]+(\*?)[\s]*([*\w]+)[\s]*\(([\w ,*.\[\]]*)\)[\s]*"
        # regex_function = "[\s]*(unsigned|signed|long|PUBLIC)*[\s]*(" + "|".join(data_types) + ")[\s]*(\*?)[\s]*([*\w]+)[\s]*\(([\s\w ,*.\[\]]*)\)[\s]*"
        #regex_function_content = "[\s]*{}[\s]*\{[\s\S]*\}[\s]*

        functions = []
        # files = self.ide.get_files_to_explore()
        ignore_spaces = bool(len(self.files) > 1)

        for file_ in self.files:

            filename = file_["filename"]
            content = file_["content"]


            # content_comments = content

            content_no_comments = remove_comments(content, ignore_spaces=ignore_spaces)
            content_list = remove_comments(content).split("\n")

            for line in range(len(content_list)):

                match = re.match(regex_function, content_list[line])
                if match:
                    this_function = {}

                    # content_no_comments = editor.text_edit.toPlainText()
                    index =  content_no_comments.find(content_list[line])
                    index_start = index
                    index_end = index
                    bracket = 1

                    try:
                        while  content_no_comments[index_end] != "{": index_end += 1
                    except IndexError:
                        bracket = 0

                    while bracket != 0:
                        index_end += 1
                        try:
                            if  content_no_comments[index_end] == "{": bracket += 1
                            elif  content_no_comments[index_end] == "}": bracket -= 1
                        except IndexError:
                            index_end = index_start - 1
                            break

                    count =  content_no_comments[index_start:index_end+1].count("\n")

                    # this_function["content"] = content_comments[index_start:index_end+1]
                    # this_function["line"] = (str(line + 1), str(line + 1 + count))
                    this_function["line"] = line + 1
                    this_function["name"] = match.groups()[2] + match.groups()[3]
                    this_function["return"] = ("" if not match.groups()[2] else "pointer to ") + ((match.groups()[0] + " " + match.groups()[1]) if match.groups()[0] else match.groups()[1])
                    this_function["return_true"] = ((match.groups()[0] + " " + match.groups()[1]) if match.groups()[0] else match.groups()[1])
                    this_function["return_true"] = this_function["return_true"].replace("PUBLIC ", "")
                    this_function["args"] = match.groups()[4]
                    this_function["filename"] = filename
                    functions.append(this_function)

        return functions
